log("Battle JS Loaded");

var Battle = function(next_monster) {
  //General order of a battle:
  // Take input from user 
  // Execute the attack
  // Check if enemy is dead (end battle win)
  // Execute enemy attack
  // Check if player is dead (end battle loss)
  // GOTO 1
  
  var me = this;
  me.activemino = null;
  me.party = [];
  me.enemy  = next_monster;
  me.active = null; // Either "player" or "enemy"
  me.busy = false;
  me.over = false;
  me.ending = false;
  me.catching = false;
  me.check_status_interval = setInterval(me.check_status, 2000);
  
  me.first_health_update = {};
  me.last_health_value = {};
  me.animating_health = false;
  
  me.energy = 0;
  
  me.ability_to_use = 'standard';
  
  me.update_health = function (target,health_left,should_animate, callback) {
    if (me.first_health_update[target] === undefined)  {
      me.last_health_value[target] = health_left;
    }
    
    var _should_animate = (me.first_health_update[target] !== undefined) && (health_left - me.last_health_value[target] > 0)
    
    should_animate = should_animate && _should_animate;
    $(target).animate({width: health_left + '%'}, !should_animate ? 0 : Math.abs(2000*(health_left - me.last_health_value[target])/100),'linear',callback);
    me.first_health_update[target] = true;
    me.last_health_value[target] = health_left;
  }
  
  me.update_enemy_health = function() {
    var health_left = 100 - Math.round(100 * me.enemy.health / me.enemy.max_hp) ;
    me.update_health('#enemy_health',health_left,true);
  }
  
  me.animate_enemy_death = function() {
    $('#monster-enemy').animate({ ypos: 110, opacity:0, fastAnimate:true }, 500);
  }
  
  me.load_player_monster = function(i) {
    me.activemino = i;
    me.display_player_monster();


    if (me.get_active_monster().dead()) {
      me.auto_switch_mino();
    }
  }
  
  me.display_player_monster = function() {
    var active_monster = me.get_active_monster();
    $('#player_monster_img').attr("src", active_monster.img());
    me.update_player_health();
  }
  
  me.animating_health = false;
  me.update_player_health = function(callback) {
    var active_monster = me.get_active_monster();
    var health_left = 100 - Math.round(100 * active_monster.health / active_monster.max_hp) ;
    me.animating_health = true;
    me.update_health('#player_health',health_left,true,function () { me.animating_health = false; if (callback !== undefined) callback();});
    $('#player_health_'+me.activemino).css('width', health_left + '%');
  }
  
  
  me.get_active_monster = function() {
    return me.party[me.activemino];
  }

  me.attack = function(from_click) {
    
    if (!_app.player.ftue["collected_doobers"] && _app.player.ftue["attacked_once"] && me.active == "player") return;
    
    
    if (!me.over && (!from_click || (from_click && !me.busy))) {
      me.busy = true;
      
      if(me.enemy.dead()) {
        me.finish();
        return;
      }
      
      if (!_app.player.ftue["attacked_once"]) {
        _app.player.track_ftue_step("attacked_once")
        $('#monster-enemy').css('z-index', '');
        $('#ftue_first_battle').fadeOut('fast');
        $('#battle_ftue_doobers').delay(2000).fadeIn('fast');
      } else if (me.active == "player" && !_app.player.ftue["attacked_twice"]) {
        _app.player.track_ftue_step("attacked_twice")
      }
      
      if (me.enemy.health/me.enemy.max_hp < 0.5 && me.active !== "player" && _app.player.ftue["started_battle_two"] && !_app.player.ftue["candy_dropped"]) {
        _app.player.track_ftue_step("candy_dropped");
        
        var endx = 390;
        var endy = 160;
        new Doober({
          "startx": 390, 
          "starty": 140, 
          "endx": 242, 
          "endy": 201,
          "destx": 110,
          "desty": 200,
          "parent_node": '#battle-field', 
          "auto_kill": false,
          "context": "battle",
          "type": "candy"
          });
        // queue the ftue text
        $('#ftue_pickup_candy').fadeIn(500);
        return;
      }
      if (me.active === "player") {
        var anim_time = me.animate_player_attack();        
        setTimeout(me.attack, anim_time + 500);
      } else {
        var anim_time = me.animate_enemy_attack();
        setTimeout(me.take_hit, anim_time)
      }
      
      setTimeout(me.shift, 1500);
    }
  }
  
  me.take_hit = function() {
    if (!me.countered) {
      var player_monster = me.get_active_monster();
      player_monster.take_damage( calculate_damage(Attacks.standard, me.enemy, player_monster) );
      var health_left = me.update_player_health(function() {
        // To be ran when health bar empties out
        //Did you die?
        if (player_monster.dead()) {
          // lets put in a sad little animation
          $('#monster-player .avatar').animate({opacity: 0},10).delay(100)
          .animate({opacity: 1.0},10).delay(100)
          .animate({opacity: 0.0},10).delay(100)
          .animate({opacity: 1.0},10).delay(100)
          .animate({opacity: 0.0},10).delay(100)
          .animate({opacity: 1.0},10).delay(100)
          .animate({top: 300},500,'linear').animate({opacity: 0, top: 15},1, function () {
            $('#monster-player').css('opacity',0);
            $('#battle').append('<div class="party_switch"><span>' + player_monster.name().toUpperCase() + ' FAINTED!</span></div>');
            $('.party_switch').delay(1000).fadeOut(500, function () {
              me.switch_mino(false);
            });

          })
        }
      });
    } else {
      me.countered = false;
    }

  }
  
  me.shift = function() {
    if (me.active === "player") {
      me.activate_enemy();
    } else {
      me.activate_player();
      me.busy = false;
    }
    _app.save();
  };
  
  me.finish = function() {
    if (!_app.player.ftue["finished_battle_one"])
      _app.player.track_ftue_step("finished_battle_one");
    
    _app.player.save();
    me.animate_enemy_death();
    me.over = true;
  }
  
  me.on_doober_pickup = function() {
    if ( !_app.player.ftue["collected_doobers"] && Doobers.count === 0 ) {
      _app.player.track_ftue_step("collected_doobers");
      $('#battle_ftue_doobers').fadeOut('fast');
    }
    
    me.check_status();
  }
  
  me.destroy = function() {
    clearInterval(me.check_status_interval);
    $('#monster-enemy').attr('style', '');
    $('#battle-field .doober').remove();
  };

  //A monster died, pull out a new one!
  me.auto_switch_mino = function() {
    me.switch_mino(true)
  }
  
  me.switch_mino = function (auto) {
    var monsters_left = false;
    for ( var i=0; i<me.party.length; i++ ){
      //Switch to another mino that's alive
      if (i != me.activemino && !me.party[i].dead() ) {
        monsters_left = true;
        if (auto)
          me.swapmino(_app.player.monsters[i].name(), _app.player.monsters[i].img(), i);
        break;
      } 
    }
    //NO MONSTERS LEFT. YOU ARE DEAD.
    if ( monsters_left == false ) {
      return battle.end('lost');
    }
    // there are monsters left present the UI
    if (!auto)
      battle.present_party();
  }

  me.swapmino = function(newtitle,newimage, id) {
    me.first_health_update = false;

    if ( me.party[id].dead()) {
      alert('this monster has fainted');
    } else {
      $('#battle').append('<div class="party_switch"><span>GO ' + newtitle.toUpperCase() + '!!!</span></div>');
      
      var src = newimage;
      
      me.load_player_monster(id);
      
      $('.monster-active').removeClass("monster-active");
      $('.monster'+id).addClass("monster-active");
      
      $('#monster-player .avatar').css('opacity',1.0);
      $('#monster-player').css('opacity',1.0);
      
      $('.party_switch').delay(1000).fadeOut('fast');
      $('#battle-field').stop(true).animate({ypos:0, fastAnimate:true},600);
      
      $('#monster-player').css('opacity',1);
      $('#monster-player .avatar').css('opacity',1.0);
      
      me.populate_skill_bar();
      me.generate_party_ui();
      flog(id);
    }
  }
  
  me.populate_skill_bar = function () {
    var skills = select(me.get_active_monster().skills, function (i) {return i.learned;});
    
    if (skills.length > 1) {
      
      $('.battle_energy').show();
      $('.energy_img').show();
      $('#battle_skill_bar').show();
      
      var skill_bar = $('#battle_skill_bar');
      skill_bar.empty();
      $.each(skills, function (i, skill) {
        if (i == 0) return;
        skill_bar.append('<div class="battle_skill_icon" id="battle_skill_icon_' + skill.name + '"><img src="i/attacks/' + skill.name + '.png" />');
        
        $('#battle_skill_icon_' + skill.name).bind(DEFAULT_HANDLER, function () {
          if (me.get_active_monster().energy >= skill.energy_cost && !me.animating_health && !me.get_active_monster().dead() && !me.busy && !me.catching) {
            $('#battle_skill_icon_' + skill.name + ' IMG:last').animate({opacity: 0.0}, 200)
            .animate({opacity: 1.0}, 200)
            .animate({opacity: 0.0}, 200)
            .animate({opacity: 1.0}, 200)
            .animate({opacity: 0.0}, 200)
            .animate({opacity: 0.35}, 200);
            me.ability_to_use = skill.name;
            me.attack(true);
            me.ability_to_use = 'standard';
          }
        });
      });
      me.update_energy();
    } else {
      $('.battle_energy').hide();
      $('.energy_img').hide();
      $('#battle_skill_bar').hide();
    }
    
    

  }
  
  me.update_energy = function () {
    $('.battle_icon_power_bar').css('background-color','');
    
    var monster = me.get_active_monster();
    var skills = select(monster.skills, function (i) {return i.learned;});
    
    $.each(skills, function (i, skill) {
      var skill_node = '#battle_skill_icon_' + skill.name;
      
      $(skill_node + ' IMG:last').stop();
      
      if (monster.energy < skill.energy_cost){
        $(skill_node).css('background-color', '#000');
        $(skill_node + ' IMG:last').css('opacity', 0.35);
      } else {
        $(skill_node).css('background-color', '#ebda3c');
        $(skill_node + '  IMG:last').css('opacity', 1.0);
      }
      
    });
    
    // I basically hand-selected these values. - TJ
    // They're pretty ok I guess. - JE
    var energy_checkpoints = [0,30,60,105,113,121,130,180,184,188,192,196,200,204,209];
    var energy_left = energy_checkpoints[ monster.energy ];
    $('.battle_energy_left').css('width',energy_left*180/209);
  }
  
  me.update_candy = function () {
    
    $('#battle .candy_button').css('display', _app.player.inventory['candy'] === undefined ? 'none' : 'block');
    
    if (_app.player.inventory['candy'] === undefined) {
      return;
    }
    
    $('#candy_count').html(_app.player.inventory['candy']);
    
    var is_dim = _app.player.inventory['candy'] == 0 || _app.player.monsters.length == 4;
    $('#battle .candy_button').css('opacity', is_dim ? 0.6 : 1.0);
  }
  
  me.add_to_party = function(monster) {
    me.party.push(monster);
  };
  
  me.activate_player = function() {
    me.active = "player";
    $('#player_avatar').addClass('active').removeClass('passive');
    $('#enemy_avatar').removeClass('active').addClass('passive');
  };
  
  me.activate_enemy = function() {
    me.active = "enemy";
    $('#enemy_avatar').addClass('active').removeClass('passive');
    $('#player_avatar').removeClass('active').addClass('passive');
  };
  
  
  me.animate_player_attack = function() {
    
    var node = $('#monster-player');
    
    var player_monster = me.get_active_monster();
    
    var top_origin = node.offset().top;
    var left_origin = node.offset().left;
    
    var attack = Attacks[me.ability_to_use];
    me.enemy.take_damage( calculate_damage(attack, player_monster, me.enemy) );
    me.get_active_monster().energy -= attack.energy_cost;
    
    me.update_energy();
    me.update_candy();
    
    var anim_time = AttackAnimations[attack.name].call(this, node, function() { me.update_enemy_health(); });
    
    
    if ( me.ability_to_use != 'standard' && element_strong_against( player_monster.element, me.enemy.element ) ) {
      setTimeout(function(){me.play_element_bonus_animation(player_monster.element)}, anim_time);
    }
    // } else if ( element_weak_against( player_monster.element, me.enemy.element ) ) {
      // me.play_element_bonus_animation(player_monster.element);
    // }
    setTimeout(function() {
      node.animate({ ypos: top_origin, xpos: left_origin, scale:1.0, fastAnimate:true }, 500);
    }, anim_time + 250)
    return anim_time + 250 + 500;
  };
  
  
  me.animate_enemy_attack = function() {
    // Perform the run & attack animation
    
    var node = $('#monster-enemy');
    
    var top_origin = node.offset().top;
    var left_origin = node.offset().left;
    
    if (me.enemy.level > 2)
      var attack = pick(EnemyAttacks);
    else
      var attack = EnemyAttacks.standard;
    
    var attack_time = attack.call(this, node)
    // node.animate({ ypos: 180, xpos: 160, scale:1.3, fastAnimate:true }, 500, function() { FX.punch(); });
    
    return attack_time;
  };
  
  me.play_level_up_animation = function() {
    $('.level_up').show()
      .animate({'font-size':'35px'}, 300)
      .animate({'font-size':'48px', opacity:0.0}, 1000, function() {  
        $('.level_up').hide().css({'font-size': '20px', 'opacity':1.0}) 
      });
  }
  
  me.play_element_bonus_animation = function(element) {
    $('.element_bonus').empty().append("<img src='i/elements/"+element.name+".png' />" + element.name + " bonus");
    $('.element_bonus').show();
    $('.element_bonus').animate({ scale: 1.5, fastAnimate:true }, 300)
      .animate({ scale: 1.7, opacity:0.0, fastAnimate:true }, 1000, function(){ $('.element_bonus').hide().css({opacity:1.0, "-webkit-transform":""}); } );
  }
  
  me.pick_up_xp = function( amt ) {
    me.get_active_monster().exp_doobed += amt;
    if (me.get_active_monster().gain_xp( amt )) {
      me.first_health_update = true;
      me.play_level_up_animation();
    }
  }
  
  me.counter_attack = function() {
    if (!me.countered) {
      me.countered = true;
      $('#monster-enemy').stop(true).css('z-index', '');
      $('#counterattack').stop(true).animate({opacity: 0.0, fastAnimate:true }, 150, function() { reset_counterattack(); });
      $('#enemy_darken').stop(true).animate({opacity:0.0 }, 150, function() { $('#enemy_darken').hide() });

      $('#monster-enemy').jrumble({ duration: 500, rangeRot:0 });
      setTimeout( function () {
        $('#monster-enemy').animate({ ypos: 55, xpos: 337, scale:1.0, fastAnimate:true }, 500);
      })
      generate_battle_doobers(5);
    }
  }
  
  me.check_status = function() {
    if (me.over && Doobers.count_in_context("battle") === 0 && !me.ending) {
      me.ending = true;
      battle.end('won');
    }
  }
  
  me.generate_party_ui = function() {
    //Put the user's monster party in their user box.
     $('#user_info_holder .monster_set').empty();
     for ( var i=0; i<me.party.length; i++ ){
          var monster = _app.player.monsters[i];
          if (me.party[i].health < 1) { var x = '<div style="position:absolute; margin:2px 0 0 12px; font-weight:bold; font-size:22px;z-index:500">X</div>' } else { var x = ''; }//dead monster
          var monsterclass = (i == me.activemino) ? 'monster_active' : 'monster_inactive';
           $('#user_info_holder .monster_set').append('<div class="' + monsterclass + '">' + x + '<img src="' + _app.player.monsters[i].img() + '" /></div>');
      }
  
      
      $('#user_info_holder .name').text(me.get_active_monster().name());
      $('#user_info_holder .level').text('level ' + me.get_active_monster().level);
      $('#user_info_holder .element').empty().append("<img src='i/elements/"+me.get_active_monster().element.name+".png' />");
  }
  
  me.init = function() {
    
    
    _app.player.spend_energy();

    // Add the players monsters to the party
    $.each(_app.player.monsters, function(i, v) {
      me.add_to_party( v );
    })
    

    // This sets monster 0 as the active monster and displays it.
    if (!me.party[_app.player.active].dead()) {
      me.load_player_monster(_app.player.active);
    } else {
      me.auto_switch_mino();
    }
    
    me.generate_party_ui();
    //
    // Build Party UI
    // Party select box. Switch between your monsters.
    //
    if ( me.party.length <= 1 ) {
      $("#show_party").hide();
    } else {
      $("#show_party").show();

      var healthstyle = 10; 
      var levelstyle = healthstyle - 10;
      var activemonster = me.get_active_monster();
      $('#party-select-bar').empty();
      for ( var i=0; i<me.party.length; i++ ){
        //this needs to be re-written. Checking if monster is active (got messy):
        var health_left = 100 - Math.round(100 * me.party[i].health / me.party[i].max_hp) ;
        var monster = _app.player.monsters[i];
        var monsterclass = (i == me.activemino) ? ' monster-active' : '';
        $('#party-select-bar').append('<div id="monster-holder" class="monster' + i + monsterclass + '"><img src="' + _app.player.monsters[i].img() + '" class="partymonster" style="left:' + levelstyle + 'px;" title="' + _app.player.monsters[i].name() +'" id="'+ i +'"> \
  <div class="monsterlevel" style="left:' + levelstyle + 'px;">' + _app.player.monsters[i].level + '</div> <div class="health" style="left:' + healthstyle + 'px;"><div class="health_overlay" style="width: '+health_left+'%;" id="player_health_'+i+'"></div></div></div>');
        var healthstyle = healthstyle + 125;
        var levelstyle = healthstyle - 10;
      } 
    }
    
    //
    // Display Enemy Monster
    //
    $('#enemy_monster_img').attr("src", me.enemy.img());
    me.update_enemy_health();
    
    
    $(".partymonster").unbind().bind(DEFAULT_HANDLER, function() {
      me.swapmino($(this).attr("title"), $(this).attr("src"), $(this).attr("id"));
    });
    
    $('#monster-player .avatar').css('opacity',1.0);
    $('#monster-player').css('opacity',1.0);
    $('.party_switch').css('opacity',0.0);
    me.populate_skill_bar();
    
    $('#counterattack').unbind().bind(DEFAULT_HANDLER, function() {
      if (!me.get_active_monster().dead()) {
        me.counter_attack();
      }
    });
    
    //place the enemy image in their info box
    $('#enemy_info_holder .monster_set').html('<div class="monster_active"><img src="' + me.enemy.img() + '" /></div>');
    $('#enemy_info_holder .name').text(me.enemy.name());
    $('#enemy_info_holder .level').text('level ' + me.enemy.level);
    $('#enemy_info_holder .element').empty().append("<img src='i/elements/"+me.enemy.element.name+".png' />");
    
    me.update_candy();
    
    // This starts the battle
    me.shift();
    
  }

  me.init();
}

var BattleController = function() {
  var me = this;
  me.active = null;
  me.next_enemy = null;
  me.last_enemy = null;
  me.next_location = null;
  me.bound_event_handlers = false;
  me._presenting_party = false;
 
  me.start = function() {
    mpmetrics.track('battle_start');
    if (!_app.player.ftue["started_battle_one"])
      _app.player.track_ftue_step("started_battle_one");
    else if (!_app.player.ftue["started_battle_two"])
      _app.player.track_ftue_step("started_battle_two");
    
    
    AnimTracker.start();
    me.active = new Battle(me.next_enemy);
    me.last_enemy = me.next_enemy;
    me.next_enemy = null;
    for ( var i=0; i<_app.player.monsters.length; i++ ){
      _app.player.monsters[i].exp_doobed = 0;
    }
  }
  
  me.end = function(result) {
    
    mpmetrics.track('battle_finish', {'result':result});
    AnimTracker.finish();
    if (result == 'won') {
      _app.player.advance_progress();
      me.discover();
    } else {
      view.shift("result");
    }
 
    
    me.active.destroy();
    me.active = null;
  }
  
  me.discover = function() {
    
    var just_fought = me.last_enemy.name();
    me.party_exp();
  }

  me.party_exp = function() {
    $('#battle-win #box_content .monster_box').remove();
    $('#battle-win #box_content .monster_box .just_captured').remove();
    for ( var i=0; i<_app.player.monsters.length; i++ ){
      var monster = _app.player.monsters[i];
      if (monster.exp_doobed == 0 || monster.exp_doobed === undefined) {
        if (monster.just_captured === undefined) 
          continue; 
      }
      
      var percent_left = Math.round(_app.player.monsters[i].level_complete_percent() * 100);
      
      var monster_box = '<div class="monster_box">';
      monster_box += '<div class="name">' + monster.name() + ' <span class="plus">+' + monster.exp_doobed + ' xp</span></div>';
      monster_box += '<div class="level_star">' + monster.level + '</div>';
      monster_box += '<div class="exp_bar"><div class="exp" style="width:' + percent_left + '%"></div></div>';
      monster_box += '<img src="' + monster.img() + '" class="winmonster" />';
      monster_box += '</div>';
      $('#battle-win #box_content').prepend(monster_box);
      
      if (monster.just_captured) {
        // add a class to this monster
        $('.monster_box:first').addClass('just_captured');
        $('.monster_box:first .plus').remove();
        $('.monster_box:first .level_star').remove();
        $('.monster_box:first .exp_bar').remove();
        monster.just_captured = undefined;      
        _app.player.save();
      }
      
    }
    var go_to_location = me.next_location;
    $('#battle_win_action').unbind().bind(DEFAULT_HANDLER, function() {
      _app.map(go_to_location);
    })
    me.next_location = null;
    _app.battle_win();
  }
  
  me.queue = function(next_monster, location_id) {
    me.next_location = location_id;
    me.next_enemy = next_monster;
  }
  
  me.present_party = function (e) {
    me._presenting_party = true;
    $('#battle-field').animate({ypos:88, fastAnimate:true},600);
  }
  
  me.dismiss_party = function (e) {
    if($(e.target).is($("#show_party"))) return;
    me._presenting_party = false;
    if (!me.active.get_active_monster().dead())
      $('#battle-field').animate({ypos:0, fastAnimate:true},600);
  }
  
  me.presenting_party = function () {
    return me._presenting_party;
  }
  
  me.init = function () {
    
    
    
    
    //
    // Event Handlers
    //
    $('#monster-enemy').bind(DEFAULT_HANDLER, function(){ 
      if (!battle.presenting_party() && !battle.active.animating_health && !battle.active.get_active_monster().dead() && !battle.active.catching)  {
        battle.active.ability_to_use = 'standard'; 
        battle.active.attack(true);
        } 
      });

    //Send a monster into battle!
    
    
    if(!battle.bound_event_handlers){
      
      $("#show_party").bind(DEFAULT_HANDLER, function(e) {
        e.preventDefault();
        battle.present_party(e);
      });
      
      $("#battle-field").bind(DEFAULT_HANDLER, function(e) {
        e.preventDefault();
        battle.dismiss_party(e);
      });
      
      battle.bound_event_handlers = true;
    }
    
    $('#attack_symbol').bind(DEFAULT_HANDLER, function () { mpmetrics.track('tap_action_indicator')});
    
    $('.candy_button').unbind().bind(DEFAULT_HANDLER,function () {
      console.log("disregard candy, acquire monsters");
      
      // only throw one candy at a time.
      if (battle.active.catching) return;
      
      // Check if we have enough candy
      if (_app.player.inventory['candy'] == 0) {
          var payed = confirm("You are out of candy! Would you like to purchase candy for $1?")          
          if (payed) {
            mpmetrics.track_funnel("pay_candy",1,"prompted_1");
            payed = confirm("You will be charged $1? Proceed?")
            
            if (payed) {
              mpmetrics.track_funnel("pay_candy",2,"prompted_2");
              if (!_app.player.free_candy) {
                mpmetrics.track_funnel("pay_candy",3,"free_refill");
                alert("We've refilled your account for free!");
                _app.player.free_candy = true;
                _app.player.inventory['candy'] += 5;
                battle.active.update_candy();
                _app.player.save();
              }
            }
          }
        return;
      }
      
      // Check if the party is full
      if (_app.player.monsters.length == 4) {
        alert('Your party is full, release some monsters to catch more.');
        return;
      }
      
      // ditch the ftue
      if (_app.player.ftue["candy_dropped"] && !_app.player.ftue["candy_thrown"]) {
        $('#ftue_throw_candy').fadeOut(500);
        _app.player.track_ftue_step("candy_thrown");
      }
      
      // Throw candy
      _app.player.consume_item('candy');
      _app.player.save();
      
      battle.active.catching = true;
      
      battle.active.update_candy();
      var img = $("<img src='i/doobers/candy-1.png' class='food_drop' />");
      var xorig = $('#monster-enemy').offset().left;
      var yorig = $('#monster-enemy').offset().top
      var xdest = xorig + ($('#monster-enemy').width()  - 37.0) / 2.0 - 100;
      var ydest = yorig  + ($('#monster-enemy').height()) / 2.0 + 100;
      $('#battle').append(img);
      $('#backpack').animate({opacity: 1.0},100).delay(500).animate({opacity: 0.0}, 500);
      img.animate({ypos:[ydest, 'easeOutBounce'], xpos: xdest, fastAnimate:true}, 800, function(){
        // bounce down
        $('#monster-enemy').animate({ypos:[ydest - 70,'easeInBounce'], xpos: xdest + 30, fastAnimate:true},1000,function () {
          var monster_img_node = $('#monster-enemy');
          var from_top = $('#monster-enemy').offset().top;
          // should catch
          var level_factor = 1;
          var level_gap  = _app.player.max_monster_level() - battle.active.enemy.level + 1;
          if (level_gap < 0) level_factor = 0.05 + Math.exp(level_gap);
          
          var R = Math.floor((1 - 0.75 * (battle.active.enemy.health/battle.active.enemy.max_hp))*100*Math.random()*level_factor);
          console.log(R + ' : ' + battle.active.enemy.race.catch_rate);
          var how_close = Math.abs(R - battle.active.enemy.race.catch_rate)
          
          if (how_close < 30) {
            // one sniff
            console.log(1);
            monster_img_node.animate({ ypos: from_top - 10,   fastAnimate:true},200)
              .animate({ ypos: from_top, fastAnimate:true},300, 'easeInQuad').delay(1000)
          } 
          if (how_close < 20) {
            // two sniff
            console.log(2);
            monster_img_node.animate({ ypos: from_top - 10,   fastAnimate:true},200)
              .animate({ ypos: from_top, fastAnimate:true},300, 'easeInQuad').delay(1000)
          }
          if (how_close < 10) {
            // three sniff
            console.log(3);
            monster_img_node.animate({ ypos: from_top - 10,   fastAnimate:true},200)
              .animate({ ypos: from_top, fastAnimate:true},300, 'easeInQuad').delay(1000)
          }
          
          monster_img_node.animate({opacity: 1.0},1, function () {
            if (R >= battle.active.enemy.race.catch_rate || !_app.player.ftue["monster_caught"]) {
              // YAY
              console.log("Catch");
              
              _app.player.track_ftue_step("monster_caught");
              
              var thought_bubble = $("<img src='i/home-thought-happy.png' class='thought_bubble' />");
              thought_bubble.css('left', $('#monster-enemy').offset().left + 50);
              $('#battle').append(thought_bubble);
              thought_bubble.delay(1000).animate({opacity:0}, 300, function() { thought_bubble.remove(); });

              monster_img_node.animate({ ypos: from_top - 35,   fastAnimate:true},200)
                .animate({ ypos: from_top, fastAnimate:true},300, 'easeInQuad')
                .animate({ ypos: from_top - 25, fastAnimate:true},150)
                .animate({ ypos: from_top, fastAnimate:true},250, 'easeInQuad')
                .animate({ ypos: from_top - 15, fastAnimate:true},100)
                .animate({ ypos: from_top,  fastAnimate:true},200, 'easeInQuad')
                .animate({ ypos: 300, xpos: 20, scale: 0.1, opacity: 0.0, fastAnimate:true},1000, function () {
                  img.animate( { scale: 2.0, opacity: 0.0, fastAnimate:true }, 200, function () {
                    img.remove();
                  })
                  // Here be the final catch code
                  mpmetrics.track('throw_candy', {did_catch: true});
                  var just_fought = battle.last_enemy.name();
                  
                  var name = prompt("Name your monster:", just_fought);
                  
                  if (!_app.player.ftue["monster_named"]) _app.player.track_ftue_step("monster_named");
                  
                  var new_monster = _app.player.capture(just_fought);
                  new_monster.player_name = name;
                  new_monster.set_to_level(battle.active.enemy.level)
                  _app.player.save();
                  
                  new_monster.just_captured = true;
                  battle.active.catching = false;
                  mpmetrics.track('catch_mino', {name: just_fought});
                  battle.end("won");
                  
                });
            } else {
              // Nooo!
              console.log("Fail");
              mpmetrics.track('throw_candy', {did_catch: false});
              var thought_bubble = $("<img src='i/home-thought-sad.png' class='thought_bubble' />");
              thought_bubble.css('left', $('#monster-enemy').offset().left + 50);
              $('#battle').append(thought_bubble);
              thought_bubble.delay(1000).animate({opacity:0}, 300, function() { thought_bubble.remove(); });
              
              $('#monster-enemy').jrumble({duration: 500}).delay(1000).animate({ypos:[yorig,'easeInBounce'], xpos: xorig , fastAnimate:true},1000, function () {
                img.animate( { scale: 2.0, opacity: 0.0, fastAnimate:true }, 200, function () {
                  img.remove();
                  battle.active.catching = false;
                });
              });
            }
          });
        })
        
      })

      // end the battle with a win
    });

  }
  me.init();
}

var battle = new BattleController();


var EnemyAttacks = {
  
  standard: function(elem, dmg) {
    
    var node = $('#monster-enemy');
    var top_origin = node.offset().top;
    var left_origin = node.offset().left;
    
    //elem.animate({ ypos: 60, xpos: 270, scale: 0.75, fastAnimate:true }, 500, function(){ generate_battle_doobers(3); dmg.call(); });
    elem.animate({ ypos: 180, xpos: 160, scale:1.3, fastAnimate:true }, 500, function() { FX.punch(); })
    .animate({ ypos: 55, xpos: 337, scale:1.0, fastAnimate:true }, 500);
    
    return 500;
  },
  
  crit: function(elem, dmg) {
    var node = $('#monster-enemy');
    var top_origin = node.offset().top;
    var left_origin = node.offset().left;
    
    elem.animate({ ypos: 180,    xpos: 160, scale: 1.3, fastAnimate:true }, 250, function(){ FX.punch(); })
        .animate({ ypos: 180+25, xpos: 160-25}, 250)
        .animate({ ypos: 180,    xpos: 160}, 250, function() { FX.punch(); })
        .animate({ ypos: 180+25, xpos: 160-25}, 250)
        .animate({ ypos: 180,    xpos: 160 }, 250, function() { FX.punch(); })
        .animate({ ypos: 55, xpos: 337, scale:1.0, fastAnimate:true }, 500);
    return 1250;
  },

  uber: function(elem, dmg) {
    enemy_darken_animation();
    counterattack_animation();
    
    var node = $('#monster-enemy');
    var top_origin = node.offset().top;
    var left_origin = node.offset().left;
    
    elem.animate({ scale: 1.3, fastAnimate:true }, 1750)
      .animate({ ypos: 180,    xpos: 160, scale: 1.3 }, 150, function(){ FX.punch(); })
      .animate({ ypos: 180+25, xpos: 160-25}, 250) // center
      .animate({ ypos: 180,    xpos: 160}, 50, function(){ FX.punch(); })
      .animate({ ypos: 180-25, xpos: 160-50}, 250) // upper left
      .animate({ ypos: 180,    xpos: 160}, 50, function(){ FX.punch(); })
      .animate({ ypos: 180+25, xpos: 160-25}, 250) // center
      .animate({ ypos: 180,    xpos: 160}, 50)
      .animate({ ypos: 180+25, xpos: 160+25}, 250) // lower right
      .animate({ ypos: 180,    xpos: 160}, 50)
      .animate({ ypos: 180+50, xpos: 160-75}, 500)
      .animate({ ypos: 55, xpos: 337, scale:1.0, fastAnimate:true }, 500);
    return 3600;
  }
  
  
  
};

function counterattack_animation() {
  $('#counterattack').show().css('z-index', 750);
  $('#counterattack').animate({scale: 0.3, opacity: 1.0, fastAnimate:true}, 150)
    .animate({scale: 1.0, fastAnimate:true}, 1500)
    .animate({opacity: 0.0 }, 150, function() { reset_counterattack() });
}

function reset_counterattack() {
  $('#counterattack').hide().prop('style', '');
}

function enemy_darken_animation() {
  $('#monster-enemy').css('z-index', 740);
  $('#enemy_darken').show()
    .animate({ opacity:0.4 }, 1750)
    .animate({ opacity:0.0 }, 250, function() { $('#enemy_darken').hide() });
    
  $('#battle-field').jrumble({ duration: 500, rangeRot:0 });
  setTimeout(function() { $('#battle-field').animate({ ypos: 0, xpos: 0, fastAnimate:true }, 250); $('#monster-enemy').css('z-index', ''); }, 2000);
}

var AttackAnimations = {
  
  standard: function(elem, dmg) {
    elem.animate({ ypos: 60, xpos: 270, scale: 0.75, fastAnimate:true }, 500, function(){ generate_battle_doobers(3); dmg.call(); });
    return 500;
  },
  
  crit: function(elem, dmg) {
    elem.animate({ ypos: 60, xpos: 270, scale: 0.75, fastAnimate:true }, 250, function(){ generate_battle_doobers(3); dmg.call(); })
      .animate({ ypos: 60+25, xpos: 270-25}, 250)
      .animate({ ypos: 60,    xpos: 270}, 250, function() { FX.punch(); })
      .animate({ ypos: 60+25, xpos: 270-25}, 250)
      .animate({ ypos: 60,    xpos: 270 }, 250, function() { FX.punch(); });
    return 1250;
  },
  
  uber: function(elem, dmg) {
    AttackAnimations.darken();
    
    elem.animate({ scale: 1.1, fastAnimate:true }, 750)
      .animate({ ypos: 60,    xpos: 270, scale: 0.75 }, 150, function(){ generate_battle_doobers(2); dmg.call(); })
      .animate({ ypos: 60+25, xpos: 270-25}, 250) // center
      .animate({ ypos: 60,    xpos: 270}, 50, function(){ generate_battle_doobers(2) })
      .animate({ ypos: 60-25, xpos: 270-50}, 250) // upper left
      .animate({ ypos: 60,    xpos: 270}, 50, function(){ generate_battle_doobers(2) })
      .animate({ ypos: 60+25, xpos: 270-25}, 250) // center
      .animate({ ypos: 60,    xpos: 270}, 50)
      .animate({ ypos: 60+25, xpos: 270+25}, 250) // lower right
      .animate({ ypos: 60,    xpos: 270}, 50)
      .animate({ ypos: 60+50, xpos: 270-75, scale:0.85}, 500);
    
    return 2600;
  },
  
  swing: function(elem, dmg) {    
    AttackAnimations.darken();
    
    elem.animate({ scale: 1.1, fastAnimate:true }, 750)
      .animate({ ypos:[60, 'easeOutQuad'], xpos:[270, 'easeInQuad'], scale:0.7}, 150, function(){ generate_battle_doobers(4); dmg.call(); })
      .animate({ xpos:270 + 300}, 150);
    
    return 1050;
  },
  
  stomp: function(elem, dmg) {
    AttackAnimations.darken();
    
    elem.animate({ scale:1.1, fastAnimate:true }, 750)
      .animate({ ypos:[-1000, 'easeOutQuad'], xpos:[160, 'easeInQuad'], scale:0.7}, 500)
      .animate({ ypos:[60, 'easeInQuad'], xpos:[270, 'easeOutQuad']}, 500, function(){ generate_battle_doobers(4); dmg.call(); })
      .animate( {ypos:40}, 100)
      .animate( {ypos:60}, 100);
  
    return 1950;
  },

  blast: function(elem, dmg) {
    AttackAnimations.darken();

    elem.animate({ scale:1.1, fastAnimate:true }, 750)
      .animate({ ypos:[-1000, 'easeOutQuad'], xpos:[160, 'easeInQuad'], scale:0.7}, 500)
      .animate({ ypos:[60, 'easeInQuad'], xpos:[270, 'easeOutQuad']}, 500, function(){ generate_battle_doobers(4); dmg.call(); })
      .animate( {ypos:40}, 100)
      .animate( {ypos:60}, 100);

    return 1950;
  },
  
  darken: function() {
    $('#player_darken').show()
      .animate({ opacity:0.4 }, 750)
      .animate({ opacity:0.0 }, 250, function() { $('#player_darken').hide() });
      
    $('#battle-field').jrumble({ duration: 500, rangeRot:0 });
    setTimeout(function() {$('#battle-field').animate({ ypos: 0, xpos: 0, fastAnimate:true }, 250); }, 600);
  }
}

