/**
 * ScrollView - jQuery plugin 0.1
 *
 * This plugin supplies contents view by grab and drag scroll.
 *
 * Copyright (c) 2009 Toshimitsu Takahashi
 *
 * Released under the MIT license.
 *
 * == Usage =======================
 *   // apply to block element.
 *   $("#map").scrollview();
 *   
 *   // with setting grab and drag icon urls.
 *   //   grab: the cursor when mouse button is up.
 *   //   grabbing: the cursor when mouse button is down.
 *   //
 *   $("#map".scrollview({
 *     grab : "images/openhand.cur",
 *     grabbing : "images/closedhand.cur"
 *   });
 * ================================
 */
 (function() {
    function ScrollView() {
        this.initialize.apply(this, arguments)
    }
    ScrollView.prototype = {
        initialize: function(container, config) {
            // setting cursor.
            var gecko = navigator.userAgent.indexOf("Gecko/") != -1;
            var opera = navigator.userAgent.indexOf("Opera/") != -1;
            var mac = navigator.userAgent.indexOf("Mac OS") != -1;
            if (opera) {
                this.grab = "default";
                this.grabbing = "move";
            } else if (! (mac && gecko) && config) {
                if (config.grab) {
                    this.grab = "url(\"" + config.grab + "\"),default";
                }
                if (config.grabbing) {
                    this.grabbing = "url(" + config.grabbing + "),move";
                }
            } else if (gecko) {
                this.grab = "-moz-grab";
                this.grabbing = "-moz-grabbing";
            } else {
                this.grab = "default";
                this.grabbing = "move";
            }

            // Get container and image.
            this.m = $(container);
            this.i = $('#map_fullsize');
            this.isgrabbing = false;
            this.touches = [];
            // Set mouse events.
            var self = this;
            this.i.touchstart(function(e) {
                self.startgrab();
                var touch = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
                this.xp = touch.pageX;
                this.yp = touch.pageY;
                self.touches = [touch];
                return true;
            }).touchmove(function(e) {
                if (!self.isgrabbing) return true;
                var touch = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
                self.scrollTo(this.xp - touch.pageX, this.yp - touch.pageY);
                this.xp = touch.pageX;
                this.yp = touch.pageY;
                self.touches.push(touch);
                return true;
            })
            .touchend(function(e) {
                self.stopgrab();
                self.inertia();
                self.touches = null;
                return true;
            })
            .mousedown(function(e) {
                self.startgrab();
                this.xp = e.pageX;
                this.yp = e.pageY;
                self.touches = [e];
                return false;
            })
            .mousemove(function(e) {
                if (!self.isgrabbing) return true;
                self.scrollTo(this.xp - e.pageX, this.yp - e.pageY);
                this.xp = e.pageX;
                this.yp = e.pageY;
                self.touches.push(e);
                return false;
            })
            .mouseup(function(e) {
                self.stopgrab()
                self.inertia(this.xp - e.pageX, this.yp - e.pageY);
                self.touches = null;
            })
            .dblclick(function() {
                var _m = self.m;
                var off = _m.offset();
                var dx = this.xp - off.left - _m.width() / 2;
                if (dx < 0) {
                    dx = "+=" + dx + "px";
                } else {
                    dx = "-=" + -dx + "px";
                }
                var dy = this.yp - off.top - _m.height() / 2;
                if (dy < 0) {
                    dy = "+=" + dy + "px";
                } else {
                    dy = "-=" + -dy + "px";
                }
                _m.animate({
                    scrollLeft: dx,
                    scrollTop: dy
                },
                "normal", "swing");
            });
            this.centering();
        },
        inertia: function(dx,dy) {
          if (this.touches.length < 2) return;
          var x = -10*(this.touches[this.touches.length-1].pageX - this.touches[this.touches.length - 2].pageX);
          var y = -10*(this.touches[this.touches.length-1].pageY - this.touches[this.touches.length - 2].pageY);
          this.m.stop().animate({ scrollLeft: this.clampWidth(x + this.m.scrollLeft()), scrollTop: this.clampHeight(y + this.m.scrollTop())},'easeOut');
        },
        centering: function() {
            var _m = this.m;
            var w = this.i.width() - _m.width();
            var h = this.i.height() - _m.height();
            _m.scrollLeft(w / 2).scrollTop(h / 2);
        },
        startgrab: function() {
            this.isgrabbing = true;
            this.i.css("cursor", this.grabbing);
        },
        stopgrab: function() {
            this.isgrabbing = false;
            this.i.css("cursor", this.grab);
        },
        clampWidth: function (x) {
          var _m = this.m;
          var clamp = function (x,max) {
            return x < max ? x : max;
          }
          return clamp(x,parseInt($(_m).children().css('width')) - _m.width())
        },
        clampHeight: function (y) {
          var _m = this.m;
          var clamp = function (x,max) {
            return x < max ? x : max;
          }
          return clamp(y,parseInt($(_m).children().css('height')) - _m.height())
        },
        scrollTo: function(dx, dy) {
            var _m = this.m;
            var x = _m.scrollLeft() + dx;
            var y = _m.scrollTop() + dy;
            

            // scrollLeft + viewWidth = imageWidth
            // _m.scrollLeft() + _m.width() = parseInt($(_m).children().css('width'))
            _m.scrollLeft(this.clampWidth(x)).scrollTop(this.clampHeight(y));
        }
    };

    jQuery.fn.scrollview = function(config) {
        return this.each(function() {
            new ScrollView(this, config);
        });
    };
})(jQuery);
