// Class for creating doobers.  Available types: star, heart, coin
// # Doober options:
//   + startx
//   + starty
//   + endx
//   + endy
//   + type (default: random)
//   + auto_kill (default: false)
//   + parent_node
function Doober( opt ) {
  var me = this;
  
  Doobers.push(this);
  
  var doober_loot_table = [ ['star',100], ['heart',100]  ];
  
  if (_app.player.ftue["monster_caught"]) {
    doober_loot_table = doober_loot_table.concat([['candy',8]]);
  }
  
  if (opt.context == 'battle' && battle.active && battle.active.get_active_monster().skills_known() > 1) {
    doober_loot_table = doober_loot_table.concat([['energy', 100 ], ['meat',5], ['pie',5], ['cake',5], ['bug',5], ['apple',5], ['banana',5]]);
  }
  
  me.context= opt.context;
  me.type = (opt.type) ? opt.type : roll_loot_table(doober_loot_table);
  
  var chance_multiplier = 1;
  if ( opt.context == 'battle' && include(Battle_Doobers, me.type) ) {
    chance_multiplier = 2 + Math.round(battle.active.enemy.level / 2.0);
  } else if ( include(Food_Doobers, me.type) ) {
    chance_multiplier = 0;
  } else if ( me.type == 'candy' ) {
    chance_multiplier = 0;
  } else if ( opt.context == 'poop')  {
    chance_multiplier = 0;
  }
  me.quantity = roll_loot_table([ [1, 950], [2, 50 * chance_multiplier] ])
  
  
  if(me.type == "energy") {
    opt.destx = 230;
    opt.desty = 248;
  } else if (me.type === "candy") {
    opt.destx = 480;
    opt.desty = 320;
  } else if (include(Food_Doobers, me.type)) {
    opt.destx = 15;
    opt.desty = 15;
  }
  
  me.pickup_duration = (opt.pickup_duration) ? opt.pickup_duration : 800;
  
  me.img = $('<img src="i/doobers/'+me.type+'-'+me.quantity+'.png" class="doober quantity-'+me.quantity+'" />');
  me.picked_up = false;
  
  me.burst = $('<img src="i/doobers/burst.png" class="doober-burst"/>');
  me.burst.css({width:72, height: 72});
  
  me.img.css({ left: opt.startx+"px", top: opt.starty+"px"});
  me.img.bind(DEFAULT_HANDLER,function(event) { event.preventDefault(); me.pickup(); } );
  
  $(opt.parent_node).append( me.img );
  
  Doobers.show();
  
  
  // Doobers make an arch as they pop out. This calculates the peak of the arch
  var arch_x = (opt.startx + opt.endx)/2;
  var arch_y = opt.starty - 35;
  
  me.img.animate( { xpos: [ arch_x,   'easeInQuad' ],   ypos: [ arch_y,   'easeOutQuad' ], fastAnimate: true }, 200 )
        .animate( { xpos: [ opt.endx, 'easeOutQuad'],   ypos: [ opt.endy, 'easeInQuad'] },                      200 )
        .animate( { scaleX: 1.2, scaleY: 0.5, ypos: opt.endy+5}, 100)
        .animate( { scaleX: 1.0, scaleY: 1.0, ypos: opt.endy  }, 100)
  
  if( opt.auto_kill && _app.player.ftue["collected_doobers"] ) {
    me.burst_timeout = setTimeout(function() { me.blow_up(); }, 3000 + 1000 * Math.random() );
  }
  
  me.blow_up = function() {
    if(!me.picked_up) {
      me.picked_up = true;
      $(opt.parent_node).append(me.burst);
      me.burst.css( { left: opt.endx - 10, top: opt.endy - 10 } );
      me.burst.animate( {opacity: 1.0 }, 100 )
      .animate( { opacity: 0 }, 100, function () { me.destroy(); } );
    }
  }
  
  me.pickup = function() {
    // Animate the doober into the player
    if(!me.picked_up) {
      me.picked_up = true;
      if (me.burst_timeout) {
        clearTimeout(me.burst_timeout);
      }
      
      if( include(Food_Doobers, me.type) ) animate_backpack();
      
      me.img.stop(true);
      me.img.animate( { xpos: [ opt.destx, 'easeOutQuad' ], ypos: [ opt.desty, 'easeInQuad' ] }, me.pickup_duration / 2.0, function() { me.deliver_bonus(); }  )
      .animate( { scale: 2.0, opacity: 0 }, me.pickup_duration / 2.0, function() { me.destroy(); } )
    } else {
      if( me.doober_index() > 0 ) {
        console.log("doober help")
        Doobers.list[me.doober_index() - 1].pickup();
      }
      
    }
  }
  
  me.deliver_bonus = function() {
    
    if( opt.context === "battle") {
    
      if (me.type === "heart") {
        if (!battle.active.get_active_monster().dead())
          battle.active.get_active_monster().take_damage( me.quantity );
        battle.active.update_player_health();
      } else if (me.type === "star") {
        battle.active.pick_up_xp( me.quantity );
      } else if (me.type == "energy") {
        battle.active.get_active_monster().consume_energy( me.quantity );
        battle.active.update_energy();
      } else if (me.type === "candy") {
        if (_app.player.ftue["candy_dropped"] && !_app.player.ftue["candy_picked_up"]) {
          _app.player.track_ftue_step("candy_picked_up");
          _app.player.save();
          $('#ftue_pickup_candy').fadeOut(500);
          $('#ftue_throw_candy').fadeIn(500);
        }
        _app.player.grant_item(me.type);
        battle.active.update_candy();
      } else if ( include( Food_Doobers, me.type ) ) {
        _app.player.grant_item(me.type);
      } 
      
    } else {
      
      if (me.type === "heart") {
        opt.owner.take_damage( 3 );
      } else if (me.type === "star") {
        opt.owner.gain_xp( 1 );
      } else if (me.type === "candy") {
        _app.player.grant_item(me.type);
      } else if (me.type == "poop") {
        mpmetrics.track('poop_pickup');
        // have a rare chance to generate some exp doobers
        if (Math.random()*100 > 70) {
          
          var monster_obj = $(opt.owner.avatar_id + " > img");

          var xdest = monster_obj.offset().left + (monster_obj.width()  - 37.0) / 2.0;
          var ydest = monster_obj.offset().top  + (monster_obj.height() - 37.0) / 2.0;
          
          // make an experience doober
          new Doober( {
              "context"         : "home",
              "type"            : Math.round(Math.random()) ? "star" : "candy",
              "startx"          : opt.startx, 
              "starty"          : opt.starty, 
              "endx"            : opt.startx, 
              "endy"            : opt.starty, 
              "destx"           : xdest, 
              "desty"           : ydest, 
              "parent_node"     : '#home', 
              "auto_kill"       : false, 
              "pickup_duration" : 400, 
              "owner"           : opt.owner
              });
        }
      }
     _app.player.save(); 
    }
  }
  
  me.doober_index = function() {
    return Doobers.list.indexOf(me);
  }
  
  me.destroy = function() {
    Doobers.pop(me.doober_index());
    me.img.remove();
    me.burst.remove(); 
    if (battle.active)
      setTimeout(battle.active.on_doober_pickup, 100);
    me = null;
  }  
}


// This keeps a count of all doobers active on the field and those queued up for display
var Doobers = {
  count: 0,
  visible: 0,
  list: [],
  push: function(d) { this.count += 1; this.list.push(d); },
  pop: function(i) { this.count -= 1; this.visible -= 1; this.list.splice(i, 1); },
  show: function() { this.visible += 1; },
  count_in_context: function(context) {
    return count_if(this.list, function(x) { return (x.context == context); });
  }
}


function generate_battle_doobers(doobers) {
  FX.punch();
  var i=0;
  var max = doobers;
  var xoff = rand(-25, 25);
  var yoff = rand(-25,25);
  var interval = setInterval(function() {
    var dooberPlacements = [[-80,15], [-65,30], [-40,30], [-15,35], [10,35]];
    var endx = 390 + xoff + dooberPlacements[i][0] + rand(-10, 10);
    var endy = 160 + yoff + dooberPlacements[i][1] + rand(-10, 10);
    new Doober({
      "startx": 390, 
      "starty": 140, 
      "endx": endx, 
      "endy": endy,
      "destx": 110,
      "desty": 200,
      "parent_node": '#battle-field', 
      "auto_kill": true,
      "context": "battle"});
    i += 1;
    if (i >= max) clearInterval(interval);
  }, 150);
}

function generate_homescreen_doobers(num_doobers, origin_monster) {
  var i=0;
  var max = num_doobers
  var xoff = 0;
  var yoff = 30;
  var doober_placements = [[-35, 0], [0, 10], [35, 0]];
  
  var monster_obj = $(origin_monster.avatar_id + " > img");
  
  var xdest = monster_obj.offset().left + (monster_obj.width()  - 37.0) / 2.0;
  var ydest = monster_obj.offset().top  + (monster_obj.height() - 37.0) / 2.0;
  
  var xorig = xdest;
  var yorig = ydest + 30;
  
  
  
  var interval = setInterval(function() {
    var placement = pick(doober_placements);
    var endx = xorig + xoff + rand(-10, 10) + placement[0];
    var endy = yorig + yoff + rand(-10, 10) + placement[1];
    new Doober({"startx":xorig, "starty":yorig, "endx":endx, "endy":endy, "destx":xdest, "desty":ydest, "parent_node":'#home', "auto_kill":false, "pickup_duration":400, "owner":origin_monster});
    i += 1;
    if (i >= max) clearInterval(interval);
  }, 150);
}

function animate_backpack() {
  clearTimeout(backpack_fadeout_timeout);
  backpack_fadeout_timeout = null;
  $('#backpack').animate( {opacity:1.0}, 500 );
  backpack_fadeout_timeout = setTimeout( function() {
    $('#backpack').animate({opacity:0.0}, 500);
  }, 1500);
}
var backpack_fadeout_timeout = null;
