//
// The Monster class
//
function Monster( options ) {
  var me = this;
  
  if (typeof options === "string") {
    
    // Initiating a new monster with just a type.  Everything is fresh
    this.race = Monsters[options];
    this.xp = 0;
    this.level = 1;
    this.temp_level = 1;
    this.max_hp = 10;
    this.health = 10;
    this.attack_power = 42;
    this.defense = 28;
    this.skills = [];
    this.last_sad = current_timestamp();
    this.last_hungry = current_timestamp();
    this.is_sad = false;
    this.pending_level_ups = [];
    this.candy = 0;
    this._dead = false;
    this.dead_until = -1;
    this.element = this.race.element;
    this.player_name = "";
    this.last_tick_time = current_timestamp();
    this.food_preference = pick(Food_Doobers);
    $.each(this.race.skills, function(i, v) {
      var skill = {}; 
      $.extend(skill,v);
      skill.learned = (me.level >= skill.level);
      me.skills.push(skill);
    })
    
  } else if (typeof options === "object") {
    
    // Initiating a monster from persisted state
    $.extend(this, options)
  }
  
  $.each(this.skills, function(i, v) {
    var skill = v;
    skill.learned = (me.level >= skill.level);
  })
  
  this.energy = 0;

};

Monster.prototype.name = function() {
  if (this.player_name != "") {
    return this.player_name;
  } else {
    return this.race.name; 
  }
};

Monster.prototype.max_energy = function () {
  var max_energy = 0;
  $.each(this.skills, function (i, skill) {
    if (skill.learned && skill.energy_cost > max_energy)
      max_energy = skill.energy_cost;
  });
  return max_energy;
}

Monster.prototype.consume_energy = function (amount) {
  if (this.dead()) return;
  this.energy += amount;
  if (this.energy > this.max_energy()) this.energy = this.max_energy();
}

Monster.prototype.img =  function() {  return "i/m/"+this.race.img; };

Monster.prototype.gain_xp = function(amt) {
  if (this.dead()) return false;
  var leveled_up = false;
  this.xp += amt;
  
  // Level Up
  while (this.temp_level < Levels.length && this.xp >= Levels[this.temp_level]) {
    this.temp_level += 1;
    // this.pending_level_ups.push(this.temp_level)
    this.grant_level_up();
    leveled_up = true;
  }
  
  return leveled_up;
};

Monster.prototype.grant_level_up = function() { 
  this.max_hp += 2;
  this.health = this.max_hp;
  this.attack_power += 28;
  this.defense += 28;
  this.level += 1;
  // this.pending_level_ups.shift();
  var new_level = this.level;
  $.each(this.skills, function(i, v) {
    if(!v.learned && v.level <= new_level) {
      v.learned = true;
    };
  });
};

Monster.prototype.xp_until_level_up = function() { return Levels[this.level] - Levels[this.level - 1]; }

Monster.prototype.xp_for_this_level = function() { return this.xp - Levels[this.level - 1]; }

Monster.prototype.level_complete_percent = function() { 
  if(this.pending_level_ups.length > 0) {
    return 1.0;
  } else {
    return this.xp_for_this_level() * 1.0 / this.xp_until_level_up();
  }
}

Monster.prototype.hp_percent = function() { return this.health * 1.0 / this.max_hp; }

Monster.prototype.dead = function() { return (this.health <= 0 && this._dead); }

Monster.prototype.take_damage = function(dmg_taken) {
  if (this.dead()) return;
  this.health += dmg_taken;
  if (this.health <= 0) this.kill();
  if (this.health > this.max_hp) this.health = this.max_hp;
  if (this.dead()) this.health = 0;
};

Monster.prototype.kill = function () {
  this.health = 0;
  this.energy = 0;
  this._dead = true;
  this.dead_until = current_timestamp() + 3600*2; // two effing hours
}

Monster.prototype.revive = function () {
  this.health = this.max_hp;
  this._dead = false;
  this.dead_until = -1;
}

Monster.prototype.tick  = function () {
  // this is where we check the revive state
  if (this.dead() && this.dead_until < current_timestamp()) this.revive();
  this.last_tick_time = current_timestamp();
}

Monster.prototype.skills_known = function() {
  var num = 0;
  $.each( this.skills, function(i, v) {
    if (v.learned) {
      num += 1;
    }
  });
  return num;
};

Monster.prototype.eat_candy = function() {
  if (this.candy < 5)
    this.candy += 1;
};
Monster.prototype.set_to_level = function(to_level) {
  while (this.level < to_level) {
    this.grant_level_up()
  }
  this.temp_level = to_level;
  this.xp = Levels[this.level-1];
}


var Elements = {
  normal:   { name: "normal",   strong: [], weak: [] },
  fire:     { name: "fire",     strong: ["earth", "air"], weak: ["water"] },
  earth:    { name: "earth",    strong: ["electric", "water"], weak: ["air", "fire"] },
  water:    { name: "water",    strong: ["fire"], weak: ["earth", "electric"] },
  air:   { name: "air",      strong: ["earth"], weak: ["fire"] },
  electric: { name: "electric", strong: ["water"], weak: ["earth"] }
}

function element_strong_against( lhs, rhs ) {
  return lhs.strong.indexOf(rhs.name) > -1;
}

function element_weak_against( lhs, rhs ) {
  return lhs.weak.indexOf(rhs.name) > -1;
}

var Levels = [0, 4, 10, 18, 36, 60, 90, 126, 168, 216, 270, 330, 396, 468, 546, 630];

var Attacks = {
  standard:{ name: "standard", friendly_name:"Headbutt",     level: 6, min_dmg: 1, max_dmg: 2,  power:1, energy_cost: 0},
  crit:    { name: "crit",     friendly_name:"Bash",         level: 7, min_dmg: 2, max_dmg: 3,  power:2, energy_cost: 2},
  uber:    { name: "uber",     friendly_name:"Shatter",      level: 9, min_dmg: 4, max_dmg: 6,  power:4, energy_cost: 6},
  swing:   { name: "swing",    friendly_name:"Dash",         level: 9, min_dmg: 4, max_dmg: 6,  power:4, energy_cost: 6},
  stomp:   { name: "stomp",    friendly_name:"Charge",       level: 9, min_dmg: 4, max_dmg: 6,  power:4, energy_cost: 6},
  blast:   { name: "blast",    friendly_name:"Deadly Blow",  level: 13, min_dmg: 8, max_dmg: 10, power:9, energy_cost: 14}
};

var Monsters = {
  "brywok": {name:"brywok", img:"brywok.png",        element: Elements.normal,   skills:[Attacks.standard, Attacks.crit, Attacks.uber], catch_rate: 50},
  "speable": { name:"speable", img:"speable.png",    element: Elements.air,      skills:[Attacks.standard, Attacks.crit, Attacks.swing], catch_rate: 50 },
  "squirt": { name:"squirt", img:"02_a.png",         element: Elements.air,      skills:[Attacks.standard, Attacks.crit, Attacks.swing], catch_rate: 25 },
  "zeewok": { name:"zeewok", img:"01b.png",          element: Elements.normal,   skills:[Attacks.standard, Attacks.crit, Attacks.uber], catch_rate: 50 },
  "felam": { name:"felam", img:"07a.png",            element: Elements.normal,   skills:[Attacks.standard, Attacks.crit, Attacks.uber], catch_rate: 50 },
  "rockey": { name:"rockey", img:"23a.png",          element: Elements.earth,    skills:[Attacks.standard, Attacks.crit, Attacks.stomp], catch_rate: 50 },
  "earth": { name:"earth", img:"earth.png",          element: Elements.earth,    skills:[Attacks.standard, Attacks.crit, Attacks.swing, Attacks.blast], catch_rate: 70 },
  "fire": { name:"fire", img:"fire.png",             element: Elements.fire,     skills:[Attacks.standard, Attacks.crit, Attacks.swing, Attacks.blast], catch_rate: 70 },
  "water": { name:"water", img:"water.png",          element: Elements.water,    skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 90 },
  "bigred": { name:"bigred", img:"boss4.png",        element: Elements.fire,     skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 70 },
  "calamari": { name:"calamari", img:"05.png",       element: Elements.water,    skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 50 },
  "hothoof": { name:"hothoof", img:"06_a.png",       element: Elements.fire,     skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 30 },
  "rook": { name:"rook", img:"flying.png",           element: Elements.fire,     skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 30 },
  "pigward": { name:"pigward", img:"ground.png",     element: Elements.earth,    skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 50 },
  "fengar": { name:"fengar", img:"fengar.png",       element: Elements.earth,    skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 60 },
  "rufus": { name:"rufus", img:"rufus.png",          element: Elements.water,    skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 30 },
  "squidly": { name:"squidly", img:"squid.png",      element: Elements.water,    skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 90 },
  "prickles": { name:"prickles", img:"prickles.png", element: Elements.normal,   skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 25 },
  "rambi": { name:"rambi", img:"rhino.png",          element: Elements.earth,    skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 50 },
  "flurl": { name:"flurl", img:"hampster.png",       element: Elements.normal,   skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 25 },
  "grunt": { name:"grunt", img:"baby_rhino.png",     element: Elements.normal,   skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 25 },
  "squaks": { name:"squaks", img:"eagle.png",        element: Elements.air,      skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 75 },
  "hoofer": { name:"hoofer", img:"horns.png",        element: Elements.normal,   skills:[Attacks.standard, Attacks.crit, Attacks.uber, Attacks.blast], catch_rate: 75 }
}

var Starter = [ new Monster("rook"), new Monster("fengar"), new Monster("rufus") ];

function monster_img_tag (monster) {
  return "<img src='"+monster.img()+"' id='player_monster' />";
}

function npc_monster(stub) {
  var mon = new Monster(stub.name);
  mon.set_to_level(stub.level);
  return mon;
}