//
// The Player Class
//

function Player( options ) {
  var me = this;
  
  if ( options ) {
    
    // Build player object from persisted state
    $.extend(this, options)
    this.monsters = [];
    $.each( options.monsters, function(i, v) { me.adopt(v); });
    
  } else {
    
    // Create a fresh player object
    this.version = APP_VERSION;
    this.uid = Math.random()+"-"+current_timestamp();
    this.active = 0;
    this.free_refill = false;
    this.free_candy = false;
    this.monsters = [];
    this.max_monsters = 4;
    
    this.ftue = {};
    
    this.last_energy_update = current_timestamp();
    this.energy = 10;
    this.max_energy = 10;
    this.progress = 0;
    this.current_location = null;
    this.current_enemy = null;
    this.level_mastery = [];
    this.inventory = {};
    
    $.each( World, function(i, v) { 
      var mastery_array = map(v.monsters, function() { return false; });
      me.level_mastery.push(mastery_array);
    });
  }
  
};

Player.prototype.active_monster = function() { return this.monsters[this.active] };
Player.prototype.save = function() { db.set('player', this); };
Player.prototype.adopt = function(monster) { return this.monsters.push(new Monster(monster)); };
Player.prototype.capture = function(monster) { 
  var mon = new Monster(monster);
  mon.is_sad = true;
  this.monsters.push(mon);
  return mon;
};
Player.prototype.energy_percent = function() { return this.energy * 1.0 / this.max_energy; }
Player.prototype.spend_energy = function() {
  if (this.energy > 0) {
    this.energy -= 2;
  }
}
Player.prototype.update_energy = function() {
  var now_unix = current_timestamp();
  var difference = now_unix - this.last_energy_update;
	var away_time = Math.floor(difference / 60*30);
  if (difference >= 60*30) {
    for (i=0;i<=away_time;i++)
    {
      this.last_energy_update = now_unix;
      this.energy += 2;
      if (this.energy > this.max_energy)
        this.energy = this.max_energy;
    }
  } else {
    
    flog ((60*30 - difference) + ' seconds until next energy heal');
    
  }
}
Player.prototype.can_adopt = function(monster) {
  if (this.monsters.length >= 4) return false;
  
  var ready = true;
  $.each( this.monsters, function(i, v) {
    if (v.name() === monster) {
      ready = false;
      return false;
    }
  })
  return ready;
}
Player.prototype.heal = function() {
  
}
Player.prototype.has_mastered = function(location) {
  var mastered_it = true;
  $.each(this.level_mastery[location], function(i, v){
    if (!v) {
      mastered_it = false;
    }
  });
  return mastered_it;
}
Player.prototype.advance_progress = function() {
  this.level_mastery[this.current_location][this.current_enemy] = true;
  
	if (this.current_location == this.progress) {
		this.progress += 1;
	};
	this.save();
}
Player.prototype.enemies_defeated_at = function(loc) {
  return select(this.level_mastery[loc], function(v){ return v; }).length;
}
Player.prototype.grant_item = function(item) {
  if (this.inventory[item] === undefined) {
    this.inventory[item] = 0;
  }
  this.inventory[item] += 1;
}
Player.prototype.consume_item = function(item) {
  if (this.inventory[item] === undefined || this.inventory[item] === 0) {
    return false;
  }
  this.inventory[item] -= 1;
  return true;
}

Player.prototype.max_monster_level = function () {
  var level = 0;
  $.each(this.monsters,function (i,monster) {
    level = (monster.level > level) ? monster.level : level;
  });
  return level;
}

Player.prototype.inventory_size = function() {
  var size = 0;
  $.each(this.inventory, function(item, amount){
    size += amount;
  });
  return size;
}

Player.prototype.inventory_food_size = function() {
  var size = 0;
  $.each(this.inventory, function(item, amount){
    if (include(Food_Doobers, item))
      size += amount;
  });
  return size;
}

ftue_funnel_steps = [
  "create_player", 
  "selected_mino",
  "started_battle_one",
  "attacked_once", 
  "collected_doobers", 
  "attacked_twice",
  "finished_battle_one", 
  "started_battle_two", 
  "candy_dropped",
  "candy_picked_up",
  "candy_thrown",
  "monster_caught",
  "monster_named"]

Player.prototype.track_ftue_step = function(key) {
  var funnel_step = ftue_funnel_steps.indexOf(key);
  
  if (Object.keys(this.ftue).length < funnel_step || funnel_step == -1) {
    console.log("FTUE ERROR AT ["+key+"], player.ftue: "+Object.keys(this.ftue).length+", funnel_index: "+funnel_step);
    return;
  }
  if (this.ftue[key] === undefined) {
    this.ftue[key] = true;
    mpmetrics.track_funnel("ftue", funnel_step + 1, key);
  }
}